#include <LiquidCrystal.h>
#include <DigitalInputPin.h>
#include <avr/pgmspace.h>

struct track_t {
  int start;
  int count;
  unsigned long totalMillis;
  unsigned long minMillis;
  DigitalInputPin *pin;
};

LiquidCrystal lcd(12, 11, 5, 4, 3, 2);
DigitalInputPin track1Pin(6, 1000);
DigitalInputPin track2Pin(7, 1000);
DigitalInputPin selectionPin(8, 300);
DigitalInputPin activationPin(9,300);

unsigned long startMillis = 0;
struct track_t tracks[2] = {
  {0, -1, 0, 1000000, &track1Pin},
  {0, -1, 0, 1000000, &track2Pin}
};

byte eacute[8] = {
  B00010,
  B00100,
  B01110,
  B10001,
  B11111,
  B10000,
  B01110,
  B00000
};

#define CONFIG 0
#define COUNTDOWN 1
#define RUNNING 2
#define END 3

int state = CONFIG;
int menuItemSelected = 0;
int turnsSet = 0;
int minutesSet = 0;
int startSignalCount = 2;
long startSignalMillis = 0;

void printCursor() {
  lcd.setCursor(1, menuItemSelected + 1);
  lcd.print("~");
}

void eraseCursor() {
  lcd.setCursor(1, menuItemSelected + 1);
  lcd.print(" ");
}  

void printConfigScreen() {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Configuration");
  lcd.setCursor(2,1);
  lcd.print("Tours:   ");
  if (turnsSet)
    lcd.print(turnsSet);
  lcd.setCursor(2,2);
  lcd.print("Minutes: ");
  if (minutesSet)
    lcd.print(minutesSet);
  lcd.setCursor(2,3);
  lcd.print("D"); lcd.write(0); lcd.print("marrer");
}  

void setup() {
  Serial.begin(9600);
  lcd.createChar(0, eacute);
  lcd.begin(20, 4);
  printConfigScreen();
  printCursor();
}

void startSignal1() {
  tone(10, 440, 1000);
}

void startSignal2() {
  tone(10, 880, 1500);
}

void errorSignal() {
  noTone(10);
  for (int i=0; i<8; i++) {
    tone(10, 660, 250);
    delay(500);
  }
}

void endSignal() {
  noTone(10);
  tone(10, 880, 3000);
}

void printFloat(long val, int column, int row) {
  lcd.setCursor(column, row);
  int i = val/1000;
  if (i < 1000)
    lcd.print(" ");
  if (i < 100)
    lcd.print(" ");
  if (i < 10)
    lcd.print(" ");
  lcd.print(i);
  lcd.print(".");
  int f = (val-(1000*i));
  if (f < 100)
    lcd.print("0");
  if (f < 10)
    lcd.print("0");
  lcd.print(f);
}

void printTimes(int i, long t) {
  if (t < 1500) return; // prevents the sound from startSignal2 to be chopy because of the LCD display
  lcd.setCursor((i*10) + 6, 0);
  if (tracks[i].count > -1) { // Race has started
    lcd.print(tracks[i].count);
    if (tracks[i].start == 1) { // Currently running
      printFloat(t - tracks[i].totalMillis, i*10, 1);
      printFloat(t, i*10, 2);
    }
    else { // Ended
      lcd.setCursor(i*10, 1);
      lcd.print("        ");
      printFloat(tracks[i].totalMillis, i*10, 2);
    }
  }
  /*if (tracks[i].start == 0 && tracks[i].count > -1)
    printFloat(tracks[i].totalMillis, i*10, 2);
  else
    printFloat(t, i*10, 2);
  */
  if (tracks[i].minMillis < 1000000)
    printFloat(tracks[i].minMillis, i*10, 3);
}

void loop() {
  if (state == CONFIG) {
    if (selectionPin.raisingEdge()) {
      eraseCursor();
      menuItemSelected = ++menuItemSelected % 3;
      printCursor();
    }
    else if (activationPin.raisingEdge()) {
      switch (menuItemSelected) {
        case 0:
          minutesSet = 0;
          lcd.setCursor(11, 2);
          lcd.print("   ");
          
          turnsSet ++;
          lcd.setCursor(11, 1);
          lcd.print(turnsSet);
          break;
        case 1:
          turnsSet = 0;
          lcd.setCursor(11, 1);
          lcd.print("   ");
          
          minutesSet ++;
          lcd.setCursor(11, 2);
          lcd.print(minutesSet);
          break;
        case 2:
          state = COUNTDOWN;
          startSignalMillis = millis();
          startSignal1();
          break;
      }     
    }
  }
  else if (state == COUNTDOWN) {
    if (millis() - startSignalMillis >= 1500) {
      if (startSignalCount > 0) {
        startSignalCount --;
        startSignalMillis = millis();
        startSignal1();
      }
      else {
        state = RUNNING;
        startMillis = millis();
        for (int i = 0; i < 2; i ++)
          tracks[i].start = 1;
        startSignal2();
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Trk1:");
        lcd.setCursor(10, 0);
        lcd.print("Trk2:");
      }
    }
  }
  
  
  long time = millis() - startMillis;
  for (int i = 0; i < 2; i ++) {
    if(tracks[i].pin->fallingEdge()) {
      if (state == RUNNING && tracks[i].start < 2) {
        int interval = time - tracks[i].totalMillis;
        tracks[i].count ++;
        if (tracks[i].count > 0 && interval < tracks[i].minMillis)
          tracks[i].minMillis = interval;
        tracks[i].totalMillis = time;
      }
      else if (state == COUNTDOWN) {
        state = CONFIG;
        startMillis = 0;
        startSignalMillis = 0;
        startSignalCount = 2;
        lcd.clear();
        lcd.setCursor(0, 1);
        lcd.print("Faux d");lcd.write(0);lcd.print("part !");
        errorSignal();
        printConfigScreen();
        printCursor();
        break;
      }  
    }
  }
  int playEndSignal = 0;
  if (state == RUNNING) {
    for (int i = 0; i < 2; i ++) {
      if (tracks[i].start == 1) {
        // Number of minutes race
        if (minutesSet && time >= minutesSet * 60000) {
          for (int j=0; j<2; j++)
            tracks[j].start = 2;
          playEndSignal = 1;
          state = END;
          break; // no need to go through the second iteration
        }
        // Number of laps race
        else if (turnsSet && tracks[i].count == turnsSet) {
          tracks[i].start = 2;
          playEndSignal = 1;
          if (tracks[i ? 0 : 1].start == 2)
            state = END;
        }
      }
    }
  }
  for (int i = 0; i < 2; i ++)
    if (tracks[i].start == 1)
      printTimes(i, time);
  if (playEndSignal) {
    for (int i = 0; i < 2; i ++)
      printTimes(i, time);
    endSignal();
  }
  
  if (state != CONFIG) {
    if (selectionPin.raisingEdge() || activationPin.raisingEdge()){
       // reset 
       state = CONFIG;
       for (int i = 0; i < 2; i++) {
         tracks[i].start = 0;
         tracks[i].count = -1;
         tracks[i].totalMillis = 0;
         tracks[i].minMillis = 1000000;
       }
       startSignalCount = 2;
       startSignalMillis = 0;
       startMillis = 0;       
       printConfigScreen();
       printCursor();
    }
  }
}

// vim:ft=C
